// primitive graphics for Hello World sce

#include <pspdisplay.h>
#include <pspthreadman.h>
#include <pspaudio.h>
#include "pg.h"

#include "font.c"

#define pg_vramtop ((char *)0x04000000)
long pg_screenmode;
long pg_showframe;
static unsigned int pg_draw_buffer;

void pgSetDrawBuffer(void *frame_buf)
{
    pg_draw_buffer = (unsigned int)frame_buf;
}

char *pgGetVramAddr(unsigned long x, unsigned long y)
{
    return pg_vramtop + pg_draw_buffer + x * PIXELSIZE * 2
        + y * LINESIZE * 2 + 0x40000000;
}

/*****************************************************************************/
int pga_ready = 0;
int pga_pause = 1;
int pga_handle;

void *(*pga_callback)(void);
int pga_threadhandle;
volatile int pga_terminate = 0;


static int pgaOutBlocking(unsigned long vol1, unsigned long vol2, void *buf)
{
    if (!pga_ready) return -1;
    if (vol1 > MAXVOLUME) vol1 = MAXVOLUME;
    if (vol2 > MAXVOLUME) vol2 = MAXVOLUME;

    return sceAudioOutputPannedBlocking(pga_handle, vol1, vol2, buf);
//    return sceAudio_2(pga_handle, vol1, vol2, buf);
}

static int pga_channel_thread(int args, void *argp)
{
    void *buf;
    unsigned long vol;

    if (pga_callback) {
        while (pga_terminate == 0) {
            buf = pga_callback();
            vol = pga_pause? 0 : MAXVOLUME;
            pgaOutBlocking(vol, vol, buf);
        }
    }

    sceKernelExitThread(0);
    return 0;
}

void pgaSetChannelCallback(void *callback)
{
    pga_callback = callback;
}

int pgaInit(int samples)
{
    pga_terminate = 0;
    pga_ready = 0;
    pga_threadhandle = -1;

#if 0
    if ((pga_handle = sceAudio_3(-1, samples, OUT_FMT_STEREO)) < 0) {
        return -1;
    }
#else
    if ((pga_handle = sceAudioChReserve(-1, samples, OUT_FMT_STEREO)) < 0) {
        return -1;
    }
#endif

    pga_ready = 1; //ȂɕKv

    pga_threadhandle = sceKernelCreateThread("pgasnd0", (SceKernelThreadEntry)&pga_channel_thread, 0x12, 0x10000, 0, NULL);

    if (pga_threadhandle >= 0) {
        if (sceKernelStartThread(pga_threadhandle, 0, NULL) == 0) {
            return 0;
        }
        sceKernelDeleteThread(pga_threadhandle);
    }
    pga_threadhandle = -1;
    pga_ready = 0;
    return -1;
}

void pgaTermPre()
{
    pga_ready = 0;
    pga_terminate = 1;
}

void pgaTerm()
{
    pga_ready = 0;
    pga_terminate = 1;

    if (pga_threadhandle != -1) {
        sceKernelWaitThreadEnd(pga_threadhandle, NULL);
        sceKernelDeleteThread(pga_threadhandle);
        pga_threadhandle = -1;
    }

    if (pga_handle != -1) {
        sceAudioChRelease(pga_handle); //sceAudio_4(pga_handle);
    }
}

void pgaPause(int n)
{
    pga_pause = n? 1 : 0;
}

//obt@64oCgEȂĂv݂
//[0]A[1]E
//Tvx44100
//vol1

